"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = configureServiceWorker;

var _workboxRouting = require("workbox-routing");

var _workboxCore = require("workbox-core");

var _workboxPrecaching = require("workbox-precaching");

var _workboxStrategies = require("workbox-strategies");

var _workboxExpiration = require("workbox-expiration");

var _workboxCacheableResponse = require("workbox-cacheable-response");

/**
 * Configures prefetching and caching of static assets as well as caching of api requests
 * in the service worker.
 *
 * **Example**
 *
 * ```js
 *  import { configureServiceWorker } from 'react-storefront/sw'
 *
 *  const maxAgeSeconds = 60 * 60 // 1 hour
 *
 *  configureServiceWorker({
 *    api: [
 *      { path: '/api/[version]/p/[productId]', maxAgeSeconds },
 *      { path: '/api/[version]/s/[subcategoryId]', maxAgeSeconds },
 *      { path: '/api/[version]/', maxAgeSeconds },
 *    ],
 *  })
 * ```
 *
 * @param {Object} config
 * @param {Object} config.api An array of objects with the following properties:
 * @param {Object} config.api.maxAgeSeconds The time to live in seconds for api requests
 * @param {Object} config.api.paths The api paths to cache
 * @param {Object} config.api.statuses Only responses with these statuses will be cached. Defaults to only caching 200s.
 */
function configureServiceWorker(config) {
  (0, _workboxCore.skipWaiting)();
  (0, _workboxCore.clientsClaim)();
  (0, _workboxPrecaching.precacheAndRoute)(self.__WB_MANIFEST || []);

  if (config.api) {
    cacheAPIRequests(config.api);
  }
}
/**
 * Creates worbox routes for api routes
 * @param {Object[]} api
 */


function cacheAPIRequests(api) {
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = api[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step$value = _step.value,
          path = _step$value.path,
          maxAgeSeconds = _step$value.maxAgeSeconds,
          _step$value$statuses = _step$value.statuses,
          statuses = _step$value$statuses === void 0 ? [200] : _step$value$statuses;
      var url = new RegExp("".concat(self.origin).concat(nextRouteToRegex(path)), 'i');
      log('Caching API route', path, url);
      (0, _workboxRouting.registerRoute)(url, // we need to remove the ^ or requests will never match
      new _workboxStrategies.CacheFirst({
        cacheName: 'api',
        plugins: [new _workboxCacheableResponse.CacheableResponsePlugin({
          statuses: statuses
        }), new _workboxExpiration.ExpirationPlugin({
          maxAgeSeconds: maxAgeSeconds
        })]
      }));
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator["return"] != null) {
        _iterator["return"]();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}
/**
 * Converts next clean route syntax to a regular expression
 * @param {String} route A next.js route pattern
 * @return {String}
 */


function nextRouteToRegex(route) {
  return route.replace(/\[[^\]]+\]/gi, '[^/]+') + '($|\\?.*$)';
}
/**
 * Stylized console.log
 * @param  {...any} message
 */


function log() {
  // istanbul ignore else
  if (process.env.NODE_ENV !== 'production') {
    var _console;

    for (var _len = arguments.length, message = new Array(_len), _key = 0; _key < _len; _key++) {
      message[_key] = arguments[_key];
    }

    (_console = console).log.apply(_console, ['%creact-storefront service-worker', 'background: #43a047; color: #ffffff; font-weight:bold; padding: 3px 5px; border-radius: 5px'].concat(message));
  }
}
//# sourceMappingURL=configureServiceWorker.js.map